/*
 *	dump.c: part of xd(heXadecimal Dump tool)
 *	Copyright  1994,95  씎Y(oo), HGG00736@niftyserve.or.jp
 */
#ifdef	HAVE_CONFIG_H
#include	"config.h"
#endif
#include	<stdio.h>
#include	<math.h>
#include	"xd.h"

#define	_m_strcpy(dst,src)	\
({\
	char* _local_dst_ = (char*)(dst);\
	char* _local_src_ = (char*)(src);\
	while (*_local_dst_++ = *_local_src_++);\
})


#ifndef	HAVE_ASM_C16TOHA
uc_t*	c16toha(uc_t* cset, uc_t* dst, uc_t* src, int n)
{
	int	c, i;

	i = min(16, n);
	while (--i >= 0) {
		c = *src++;
		*dst++ = cset[c >>  4];
		*dst++ = cset[c & 0xf];
		*dst++ = ' ';
	}
	if ((i = 16 - n) > 0) {
		while (--i >= 0) {
			*dst++ = ' ';
			*dst++ = ' ';
			*dst++ = ' ';
		}
	}
	*dst++ = ' ';
	return dst;
}
#endif

#ifndef	HAVE_ASM_C16TODA
uc_t*	c16toda(uc_t* cset, uc_t* dst, uc_t* src, int n)
{
	int	c, i;
	int	sign;
	char*	p = (char*)src;

	i = min(16, n);
	while (--i >= 0) {
		sign = '+';
		if ((c = *p++) < 0) {
			sign = '-';
			c = -c;
		}
		sprintf(dst, "%c%03u ", sign, c);
		dst += 5;
	}
	if ((i = 16 - n) > 0) {
		while (--i >= 0) {
			*dst++ = ' ';
			*dst++ = ' ';
			*dst++ = ' ';
			*dst++ = ' ';
			*dst++ = ' ';
		}
	}
	*dst++ = ' ';
	return dst;
}
#endif

#ifndef	HAVE_ASM_C16TOUA
uc_t*	c16toua(uc_t* cset, uc_t* dst, uc_t* src, int n)
{
	int	c, i;

	i = min(16, n);
	while (--i >= 0) {
		c = *src++;
		sprintf(dst, " %03u ", c);
		dst += 4;
	}
	if ((i = 16 - n) > 0) {
		while (--i >= 0) {
			*dst++ = ' ';
			*dst++ = ' ';
			*dst++ = ' ';
			*dst++ = ' ';
		}
	}
	*dst++ = ' ';
	return dst;
}
#endif

#ifndef	HAVE_ASM_B2TOHA
uc_t*	b2toha(uc_t* cset, uc_t* dst, uc_t* src, int n)
{
	int	c, i;

	i = min(16, n) >> 1;
	while (--i >= 0) {
		c = *src++;
		*dst++ = cset[c >>  4];
		*dst++ = cset[c & 0xf];
		c = *src++;
		*dst++ = cset[c >>  4];
		*dst++ = cset[c & 0xf];
		*dst++ = ' ';
		*dst++ = ' ';
	}
	if ((i = 16 - n) > 0) {
		i = (i + 1) & ~1;
		while (--i >= 0) {
			*dst++ = ' ';
			*dst++ = ' ';
			*dst++ = ' ';
		}
	}
	*dst++ = ' ';
	return dst;
}
#endif

#ifndef	HAVE_ASM_L2TOHA
uc_t*	l2toha(uc_t* cset, uc_t* dst, uc_t* src, int n)
{
	int	c, i;

	i = min(16, n) >> 1;
	while (--i >= 0) {
		c = src[1];
		*dst++ = cset[c >>  4];
		*dst++ = cset[c & 0xf];
		c = src[0];
		*dst++ = cset[c >>  4];
		*dst++ = cset[c & 0xf];
		*dst++ = ' ';
		*dst++ = ' ';
		src += 2;
	}
	if ((i = 16 - n) > 0) {
		i = (i + 1) & ~1;
		while (--i >= 0) {
			*dst++ = ' ';
			*dst++ = ' ';
			*dst++ = ' ';
		}
	}
	*dst++ = ' ';
	return dst;
}
#endif

#ifndef	HAVE_ASM_B2TODA
uc_t*	b2toda(uc_t* cset, uc_t* dst, uc_t* src, int n)
{
	short	s;
	int	i;
	int	sign;

	i = min(16, n) >> 1;
	while (--i >= 0) {
		sign = '+';
		if (endian == BIG_ENDIAN) {
			s = *(short*)src;
		}
		else {
			((uc_t*)&s)[1] = src[0];
			((uc_t*)&s)[0] = src[1];
		}
		src += 2;
		if (s < 0) {
			sign = '-';
			s = -s;
		}
		sprintf(dst, "%c%05u ", sign, s);
		dst += 7;
	}
	if ((i = 16 - n) > 0) {
		i = ((i + 1) >> 1) * 7;
		while (--i >= 0) {
			*dst++ = ' ';
		}
	}
	*dst++ = ' ';
	return dst;
}
#endif

#ifndef	HAVE_ASM_L2TODA
uc_t*	l2toda(uc_t* cset, uc_t* dst, uc_t* src, int n)
{
	short	s;
	int	i;
	int	sign;

	i = min(16, n) >> 1;
	while (--i >= 0) {
		sign = '+';
		if (endian == BIG_ENDIAN) {
			((uc_t*)&s)[0] = src[1];
			((uc_t*)&s)[1] = src[0];
		}
		else {
			s = *(short*)src;
		}
		src += 2;
		if (s < 0) {
			sign = '-';
			s = -s;
		}
		sprintf(dst, "%c%05u ", sign, s);
		dst += 7;
	}
	if ((i = 16 - n) > 0) {
		i = ((i + 1) >> 1) * 7;
		while (--i >= 0) {
			*dst++ = ' ';
		}
	}
	*dst = ' ';
	return dst;
}
#endif

#ifndef	HAVE_ASM_B2TOUA
uc_t*	b2toua(uc_t* cset, uc_t* dst, uc_t* src, int n)
{
	short	s;
	int	i;

	i = min(16, n) >> 1;
	while (--i >= 0) {
		if (endian == BIG_ENDIAN) {
			s = *(short*)src;
		}
		else {
			((uc_t*)&s)[1] = src[0];
			((uc_t*)&s)[0] = src[1];
		}
		src += 2;
		sprintf(dst, " %05u ", s);
		dst += 6;
	}
	if ((i = 16 - n) > 0) {
		i = ((i + 1) >> 1) * 6;
		while (--i >= 0) {
			*dst++ = ' ';
		}
	}
	*dst++ = ' ';
	return dst;
}
#endif

#ifndef	HAVE_ASM_L2TOUA
uc_t*	l2toua(uc_t* cset, uc_t* dst, uc_t* src, int n)
{
	short	s;
	int	i;

	i = min(16, n) >> 1;
	while (--i >= 0) {
		if (endian == BIG_ENDIAN) {
			((uc_t*)&s)[0] = src[1];
			((uc_t*)&s)[1] = src[0];
		}
		else {
			s = *(short*)src;
		}
		src += 2;
		sprintf(dst, " %05u ", s);
		dst += 6;
	}
	if ((i = 16 - n) > 0) {
		i = ((i + 1) >> 1) * 6;
		while (--i >= 0) {
			*dst++ = ' ';
		}
	}
	*dst++ = ' ';
	return dst;
}
#endif

#ifndef	HAVE_ASM_B4TOHA
uc_t*	b4toha(uc_t* cset, uc_t* dst, uc_t* src, int n)
{
	int	c, i;

	i = min(16, n) >> 2;
	while (--i >= 0) {
		c = *src++;
		*dst++ = cset[c >>  4];
		*dst++ = cset[c & 0xf];
		c = *src++;
		*dst++ = cset[c >>  4];
		*dst++ = cset[c & 0xf];
		c = *src++;
		*dst++ = cset[c >>  4];
		*dst++ = cset[c & 0xf];
		c = *src++;
		*dst++ = cset[c >>  4];
		*dst++ = cset[c & 0xf];
		*dst++ = ' ';
		*dst++ = ' ';
		*dst++ = ' ';
		*dst++ = ' ';
	}
	if ((i = 16 - n) > 0) {
		i = (i + 3) & ~3;
		while (--i >= 0) {
			*dst++ = ' ';
			*dst++ = ' ';
			*dst++ = ' ';
		}
	}
	*dst++ = ' ';
	return dst;
}
#endif

#ifndef	HAVE_ASM_L4TOHA
uc_t*	l4toha(uc_t* cset, uc_t* dst, uc_t* src, int n)
{
	int	c, i;

	i = min(16, n) >> 2;
	while (--i >= 0) {
		c = src[3];
		*dst++ = cset[c >>  4];
		*dst++ = cset[c & 0xf];
		c = src[2];
		*dst++ = cset[c >>  4];
		*dst++ = cset[c & 0xf];
		c = src[1];
		*dst++ = cset[c >>  4];
		*dst++ = cset[c & 0xf];
		c = src[0];
		*dst++ = cset[c >>  4];
		*dst++ = cset[c & 0xf];
		*dst++ = ' ';
		*dst++ = ' ';
		*dst++ = ' ';
		*dst++ = ' ';
		src += 4;
	}
	if ((i = 16 - n) > 0) {
		i = (i + 3) & ~3;
		while (--i >= 0) {
			*dst++ = ' ';
			*dst++ = ' ';
			*dst++ = ' ';
		}
	}
	*dst++ = ' ';
	return dst;
}
#endif

#ifndef	HAVE_ASM_B4TODA
uc_t*	b4toda(uc_t* cset, uc_t* dst, uc_t* src, int n)
{
	long	l;
	int	i;
	int	sign;

	i = min(16, n) >> 2;
	while (--i >= 0) {
		sign = '+';
		if (endian == BIG_ENDIAN) {
			l = *(long*)src;
		}
		else {
			((uc_t*)&l)[3] = src[0];
			((uc_t*)&l)[2] = src[1];
			((uc_t*)&l)[1] = src[2];
			((uc_t*)&l)[0] = src[3];
		}
		if (l < 0) {
			sign = '-';
			l = -l;
		}
		src += 4;
		sprintf(dst, "%c%010lu ", sign, l);
		dst += 12;
	}
	if ((i = 16 - n) > 0) {
		i = ((i + 3) >> 2) * 12;
		while (--i >= 0) {
			*dst++ = ' ';
		}
	}
	*dst++ = ' ';
	return dst;
}
#endif

#ifndef	HAVE_ASM_L4TODA
uc_t*	l4toda(uc_t* cset, uc_t* dst, uc_t* src, int n)
{
	long	l;
	int	i;
	int	sign;

	i = min(16, n) >> 2;
	while (--i >= 0) {
		sign = '+';
		if (endian == BIG_ENDIAN) {
			((uc_t*)&l)[0] = src[3];
			((uc_t*)&l)[1] = src[2];
			((uc_t*)&l)[2] = src[1];
			((uc_t*)&l)[3] = src[0];
		}
		else {
			l = *(long*)src;
		}
		if (l < 0) {
			sign = '-';
			l = -l;
		}
		src += 4;
		sprintf(dst, "%c%010lu ", sign, l);
		dst += 12;
	}
	if ((i = 16 - n) > 0) {
		i = ((i + 3) >> 2) * 12;
		while (--i >= 0) {
			*dst++ = ' ';
		}
	}
	*dst++ = ' ';
	return dst;
}
#endif

#ifndef	HAVE_ASM_B4TOUA
uc_t*	b4toua(uc_t* cset, uc_t* dst, uc_t* src, int n)
{
	long	l;
	int	i;

	i = min(16, n) >> 2;
	while (--i >= 0) {
		if (endian == BIG_ENDIAN) {
			l = *(long*)src;
		}
		else {
			((uc_t*)&l)[3] = src[0];
			((uc_t*)&l)[2] = src[1];
			((uc_t*)&l)[1] = src[2];
			((uc_t*)&l)[0] = src[3];
		}
		src += 4;
		sprintf(dst, " %010lu ", l);
		dst += 11;
	}
	if ((i = 16 - n) > 0) {
		i = ((i + 3) >> 2) * 11;
		while (--i >= 0) {
			*dst++ = ' ';
		}
	}
	*dst++ = ' ';
	return dst;
}
#endif

#ifndef	HAVE_ASM_L4TOUA
uc_t*	l4toua(uc_t* cset, uc_t* dst, uc_t* src, int n)
{
	long	l;
	int	i;

	i = min(16, n) >> 2;
	while (--i >= 0) {
		if (endian == BIG_ENDIAN) {
			((uc_t*)&l)[0] = src[3];
			((uc_t*)&l)[1] = src[2];
			((uc_t*)&l)[2] = src[1];
			((uc_t*)&l)[3] = src[0];
		}
		else {
			l = *(long*)src;
		}
		src += 4;
		sprintf(dst, " %010lu ", l);
		dst += 11;
	}
	if ((i = 16 - n) > 0) {
		i = ((i + 3) >> 2) * 11;
		while (--i >= 0) {
			*dst++ = ' ';
		}
	}
	*dst++ = ' ';
	return dst;
}
#endif

#ifndef	HAVE_ASM_B4TOFA
uc_t*	b4tofa(uc_t* cset, uc_t* dst, uc_t* src, int n)
{
	float	f;
	int	i;

	i = min(16, n) >> 2;
	while (--i >= 0) {
		if (endian == BIG_ENDIAN) {
			f = *(float*)src;
		}
		else {
			((uc_t*)&f)[3] = src[0];
			((uc_t*)&f)[2] = src[1];
			((uc_t*)&f)[1] = src[2];
			((uc_t*)&f)[0] = src[3];
		}
		src += 4;
		if (isnan((double)f)) {
			_m_strcpy(dst, "(NaN)           ");
		}
		else if (isinf((double)f)) {
			_m_strcpy(dst, "(Inf)           ");
		}
		else {
			sprintf(dst, "%+#.8e ", (double)f);
		}
		dst += 16;		/* S:1, I.:2, P:8, e+nn:4, ' ':1 = 16 */
	}
	if ((i = 16 - n) > 0) {
		i = ((i + 3) >> 2) * 16;
		while (--i >= 0) {
			*dst++ = ' ';
		}
	}
	*dst++ = ' ';
	return dst;
}
#endif

#ifndef	HAVE_ASM_L4TOFA
uc_t*	l4tofa(uc_t* cset, uc_t* dst, uc_t* src, int n)
{
	float	f;
	int	i;

	i = min(16, n) >> 2;
	while (--i >= 0) {
		if (endian == BIG_ENDIAN) {
			((uc_t*)&f)[0] = src[3];
			((uc_t*)&f)[1] = src[2];
			((uc_t*)&f)[2] = src[1];
			((uc_t*)&f)[3] = src[0];
		}
		else {
			f = *(float*)src;
		}
		src += 4;
		if (isnan((double)f)) {
			_m_strcpy(dst, "(NaN)           ");
		}
		else if (isinf((double)f)) {
			_m_strcpy(dst, "(Inf)           ");
		}
		else {
			sprintf(dst, "%+#.8e ", (double)f);
		}
		dst += 16;		/* S:1, I.:2, P:8, e+nn:4, ' ':1 = 16 */
	}
	if ((i = 16 - n) > 0) {
		i = ((i + 3) >> 2) * 16;
		while (--i >= 0) {
			*dst++ = ' ';
		}
	}
	*dst++ = ' ';
	return dst;
}
#endif

#ifndef	HAVE_ASM_B8TOHA
uc_t*	b8toha(uc_t* cset, uc_t* dst, uc_t* src, int n)
{
	int	c, i;

	i = min(16, n) >> 3;
	while (--i >= 0) {
		c = src[0];
		*dst++ = cset[c >>  4];
		*dst++ = cset[c & 0xf];
		c = src[1];
		*dst++ = cset[c >>  4];
		*dst++ = cset[c & 0xf];
		c = src[2];
		*dst++ = cset[c >>  4];
		*dst++ = cset[c & 0xf];
		c = src[3];
		*dst++ = cset[c >>  4];
		*dst++ = cset[c & 0xf];
		c = src[4];
		*dst++ = cset[c >>  4];
		*dst++ = cset[c & 0xf];
		c = src[5];
		*dst++ = cset[c >>  4];
		*dst++ = cset[c & 0xf];
		c = src[6];
		*dst++ = cset[c >>  4];
		*dst++ = cset[c & 0xf];
		c = src[7];
		*dst++ = cset[c >>  4];
		*dst++ = cset[c & 0xf];
		*dst++ = ' ';
		*dst++ = ' ';
		*dst++ = ' ';
		*dst++ = ' ';
		*dst++ = ' ';
		*dst++ = ' ';
		*dst++ = ' ';
		*dst++ = ' ';
		src += 8;
	}
	if ((i = 16 - n) > 0) {
		i = (i + 7) & ~7;
		while (--i >= 0) {
			*dst++ = ' ';
			*dst++ = ' ';
			*dst++ = ' ';
		}
	}
	*dst++ = ' ';
	return dst;
}
#endif

#ifndef	HAVE_ASM_L8TOHA
uc_t*	l8toha(uc_t* cset, uc_t* dst, uc_t* src, int n)
{
	int	c, i;

	i = min(16, n) >> 3;
	while (--i >= 0) {
		c = src[7];
		*dst++ = cset[c >>  4];
		*dst++ = cset[c & 0xf];
		c = src[6];
		*dst++ = cset[c >>  4];
		*dst++ = cset[c & 0xf];
		c = src[5];
		*dst++ = cset[c >>  4];
		*dst++ = cset[c & 0xf];
		c = src[4];
		*dst++ = cset[c >>  4];
		*dst++ = cset[c & 0xf];
		c = src[3];
		*dst++ = cset[c >>  4];
		*dst++ = cset[c & 0xf];
		c = src[2];
		*dst++ = cset[c >>  4];
		*dst++ = cset[c & 0xf];
		c = src[1];
		*dst++ = cset[c >>  4];
		*dst++ = cset[c & 0xf];
		c = src[0];
		*dst++ = cset[c >>  4];
		*dst++ = cset[c & 0xf];
		*dst++ = ' ';
		*dst++ = ' ';
		*dst++ = ' ';
		*dst++ = ' ';
		*dst++ = ' ';
		*dst++ = ' ';
		*dst++ = ' ';
		*dst++ = ' ';
		src += 8;
	}
	if ((i = 16 - n) > 0) {
		i = (i + 7) & ~7;
		while (--i >= 0) {
			*dst++ = ' ';
			*dst++ = ' ';
			*dst++ = ' ';
		}
	}
	*dst++ = ' ';
	return dst;
}
#endif

#ifdef	HAVE_LONGLONGINT
#ifndef	HAVE_ASM_B8TODA
uc_t*	b8toda(uc_t* cset, uc_t* dst, uc_t* src, int n)
{
	long long	ll;
	int	i;
	int	sign;

	i = min(16, n) >> 3;
	while (--i >= 0) {
		sign = '+';
		if (endian == BIG_ENDIAN) {
			ll = *(long long*)src;
		}
		else {
			((uc_t*)&ll)[7] = src[4];
			((uc_t*)&ll)[6] = src[5];
			((uc_t*)&ll)[5] = src[6];
			((uc_t*)&ll)[4] = src[7];
			((uc_t*)&ll)[3] = src[0];
			((uc_t*)&ll)[2] = src[1];
			((uc_t*)&ll)[1] = src[2];
			((uc_t*)&ll)[0] = src[3];
		}
		if (ll < 0) {
			sign = '-';
			ll = -ll;
		}
		src += 8;
		sprintf(dst, "%c%020Lu ", sign, ll);
		dst += 22;
	}
	if ((i = 16 - n) > 0) {
		i = ((i + 7) >> 3) * 22;
		while (--i >= 0) {
			*dst++ = ' ';
		}
	}
	*dst++ = ' ';
	return dst;
}
#endif

#ifndef	HAVE_ASM_L8TODA
uc_t*	l8toda(uc_t* cset, uc_t* dst, uc_t* src, int n)
{
	long long	ll;
	int	i;
	int	sign;

	i = min(16, n) >> 3;
	while (--i >= 0) {
		sign = '+';
		if (endian == BIG_ENDIAN) {
			((uc_t*)&ll)[0] = src[3];
			((uc_t*)&ll)[1] = src[2];
			((uc_t*)&ll)[2] = src[1];
			((uc_t*)&ll)[3] = src[0];
			((uc_t*)&ll)[4] = src[7];
			((uc_t*)&ll)[5] = src[6];
			((uc_t*)&ll)[6] = src[5];
			((uc_t*)&ll)[7] = src[4];
		}
		else {
			ll = *(long long*)src;
		}
		if (ll < 0) {
			sign = '-';
			ll = -ll;
		}
		src += 8;
		sprintf(dst, "%c%020Lu ", sign, ll);
		dst += 22;
	}
	if ((i = 16 - n) > 0) {
		i = ((i + 7) >> 3) * 22;
		while (--i >= 0) {
			*dst++ = ' ';
		}
	}
	*dst++ = ' ';
	return dst;
}
#endif

#ifndef	HAVE_ASM_B8TOUA
uc_t*	b8toua(uc_t* cset, uc_t* dst, uc_t* src, int n)
{
	long long	ll;
	int	i;

	i = min(16, n) >> 3;
	while (--i >= 0) {
		if (endian == BIG_ENDIAN) {
			ll = *(long long*)src;
		}
		else {
			((uc_t*)&ll)[7] = src[4];
			((uc_t*)&ll)[6] = src[5];
			((uc_t*)&ll)[5] = src[6];
			((uc_t*)&ll)[4] = src[7];
			((uc_t*)&ll)[3] = src[0];
			((uc_t*)&ll)[2] = src[1];
			((uc_t*)&ll)[1] = src[2];
			((uc_t*)&ll)[0] = src[3];
		}
		src += 8;
		sprintf(dst, " %020Lu ", ll);
		dst += 21;
	}
	if ((i = 16 - n) > 0) {
		i = ((i + 7) >> 3) * 21;
		while (--i >= 0) {
			*dst++ = ' ';
		}
	}
	*dst++ = ' ';
	return dst;
}
#endif

#ifndef	HAVE_ASM_L8TOUA
uc_t*	l8toua(uc_t* cset, uc_t* dst, uc_t* src, int n)
{
	long long	ll;
	int	i;

	i = min(16, n) >> 3;
	while (--i >= 0) {
		if (endian == BIG_ENDIAN) {
			((uc_t*)&ll)[0] = src[3];
			((uc_t*)&ll)[1] = src[2];
			((uc_t*)&ll)[2] = src[1];
			((uc_t*)&ll)[3] = src[0];
			((uc_t*)&ll)[4] = src[7];
			((uc_t*)&ll)[5] = src[6];
			((uc_t*)&ll)[6] = src[5];
			((uc_t*)&ll)[7] = src[4];
		}
		else {
			ll = *(long long*)src;
		}
		src += 8;
		sprintf(dst, " %020Lu ", ll);
		dst += 21;
	}
	if ((i = 16 - n) > 0) {
		i = ((i + 7) >> 3) * 21;
		while (--i >= 0) {
			*dst++ = ' ';
		}
	}
	*dst++ = ' ';
	return dst;
}
#endif
#endif	/*HAVE_LONGLONGINT*/

#ifndef	HAVE_ASM_B8TOFA
uc_t*	b8tofa(uc_t* cset, uc_t* dst, uc_t* src, int n)
{
	double	d;
	int	i;

	i = min(16, n) >> 3;
	while (--i >= 0) {
		if (endian == BIG_ENDIAN) {
			d = *(double*)src;
		}
		else {
			((uc_t*)&d)[7] = src[0];
			((uc_t*)&d)[6] = src[1];
			((uc_t*)&d)[5] = src[2];
			((uc_t*)&d)[4] = src[3];
			((uc_t*)&d)[3] = src[4];
			((uc_t*)&d)[2] = src[5];
			((uc_t*)&d)[1] = src[6];
			((uc_t*)&d)[0] = src[7];
		}
		src += 8;
		if (isnan(d)) {
			_m_strcpy(dst, "(NaN)                    ");
		}
		else if (isinf(d)) {
			_m_strcpy(dst, "(Inf)                    ");
		}
		else {
			sprintf(dst, "%+#.16e ", d);
		}
		dst += 25;		/* S:1, I.:2, P:16, e+nnn:5, ' ':1 = 25 */
		if (*(dst - 1) == '\0') {
			*(dst - 1) = *(dst - 2);	/* ' ' */
			*(dst - 2) = *(dst - 3);	/* +nN */
			*(dst - 3) = *(dst - 4);	/* +Nn */
			*(dst - 4) = '0';
		}
	}
	if ((i = 16 - n) > 0) {
		i = ((i + 7) >> 3) * 25;
		while (--i >= 0) {
			*dst++ = ' ';
		}
	}
	*dst++ = ' ';
	return dst;
}
#endif

#ifndef	HAVE_ASM_L8TOFA
uc_t*	l8tofa(uc_t* cset, uc_t* dst, uc_t* src, int n)
{
	double	d;
	int	i;

	i = min(16, n) >> 3;
	while (--i >= 0) {
		if (endian == BIG_ENDIAN) {
			((uc_t*)&d)[0] = src[7];
			((uc_t*)&d)[1] = src[6];
			((uc_t*)&d)[2] = src[5];
			((uc_t*)&d)[3] = src[4];
			((uc_t*)&d)[4] = src[3];
			((uc_t*)&d)[5] = src[2];
			((uc_t*)&d)[6] = src[1];
			((uc_t*)&d)[7] = src[0];
		}
		else {
			d = *(double*)src;
		}
		src += 8;
		if (isnan(d)) {
			_m_strcpy(dst, "(NaN)                    ");
		}
		else if (isinf(d)) {
			_m_strcpy(dst, "(Inf)                    ");
		}
		else {
			sprintf(dst, "%+#.16e ", d);
		}
		dst += 25;		/* S:1, I.:2, P:16, e+nnn:5, ' ':1 = 25 */
		if (*(dst - 1) == '\0') {
			*(dst - 1) = *(dst - 2);	/* ' ' */
			*(dst - 2) = *(dst - 3);	/* +nN */
			*(dst - 3) = *(dst - 4);	/* +Nn */
			*(dst - 4) = '0';
		}
	}
	if ((i = 16 - n) > 0) {
		i = ((i + 7) >> 3) * 25;
		while (--i >= 0) {
			*dst++ = ' ';
		}
	}
	*dst++ = ' ';
	return dst;
}
#endif

#ifndef	HAVE_ASM_B4TOADDR
uc_t*	b4toaddr(uc_t* cset, uc_t* dst, ul_t* src)
{
	int	c, cnt;
	uc_t*	p = (uc_t*)src;

	for (cnt = 4; --cnt >= 0;) {
		c = *p++;
		*dst++ = cset[c >>  4];
		*dst++ = cset[c & 0xf];
	}
	*dst++ = ' ';
	*dst++ = ' ';
	return dst;
}
#endif

#ifndef	HAVE_ASM_L4TOADDR
uc_t*	l4toaddr(uc_t* cset, uc_t* dst, ul_t* src)
{
	int	c, cnt;
	uc_t*	p = (uc_t*)src + 4;

	for (cnt = 4; --cnt >= 0;) {
		c = *--p;
		*dst++ = cset[c >>  4];
		*dst++ = cset[c & 0xf];
	}
	*dst++ = ' ';
	*dst++ = ' ';
	return dst;
}
#endif


